@@ -1,8 +1,8 @@
-#rm(list = ls(all = TRUE))
-#source("NI_PWK.R")
 
-#Set the hyperparameter of a_0
-#To reproduce Table 6, change a_0 to 0, 0.1, 0.2,..., 1.0
+# Some of the following comments appeared in the source code for 
+# Wang YB, Chen MH, Kuo L, Lewis PO (2018). “A New Monte Carlo Method for Estimating Marginal Likelihoods.” Bayesian Analysis, 13(2), 311–333.
+
+
 globalVariables(c("historical", "data.type", "data.link"))
 
 
@@ -11,11 +11,10 @@ calc_marg_l <- function(a0, mcmc){
   sds <- apply(mcmc, 2, stats::sd)
   means <- apply(mcmc, 2, mean)
 
-  # standardize MCMC samples
+  # Standardize MCMC samples
   mcmc_scaled <- scale(mcmc, center=TRUE, scale = TRUE)
 
-  #Determine r ("rcover") and K ("ncutslice") for PWK
-  #To reproduce Table 6, change "rcover" to 2.81325, 3.751, or 4.68875 and change "ncutslice" to 10, 20, or 100
+  # Determine r ("rcover") and K ("ncutslice") for PWK
   rcover <- sqrt(qchisq(0.95,df=ncol(mcmc)))
   ncutslice <- 100
 
@@ -27,8 +26,6 @@ calc_marg_l <- function(a0, mcmc){
   #Estimate c_0
   t1 <- dim(mcmc_scaled)[1]
   val <- rep(NA, t1)
-  #inside <- 0
-  #varne <- matrix(rep(NA, 2*t1), ncol=2)
   r_ <- sqrt(apply(mcmc_scaled^2, 1, sum )) # norm of mcmc samples
   kreprp <- rep(NA, t1)
   for (j in 1:t1 ){
@@ -40,22 +37,21 @@ calc_marg_l <- function(a0, mcmc){
       kreprp[j] <- 0
       kreprp[j] <- kreprp[j] + logpowerprior(mcmc[j,], a0) + partjo1
       val[j] <- rings_pp[ring_position, 3] - kreprp[j]
-      #inside <- inside + 1
-      #varne[j, ] <- c(exp(2*val[j]), ring_position)
+
     }
   }
   lcons <- max(rings_pp[, 4])
-  tcubevol <- log(sum(exp(rings_pp[,4] - lcons) ) ) + lcons #floating control?
+  tcubevol <- log(sum(exp(rings_pp[,4] - lcons) ) ) + lcons 
 
   den <- denominator_control(val)
   c_est_pp <- NA
   c_est_pp <- tcubevol - den
-  return(c_est_pp) # what i want in the end: log(c0)
+  return(c_est_pp) # log(c0)
 
 }
 
 
-
+# This function computes the normalizing constant for a given a_0 value. 
 calc_a0_func <- function(a0, historical, data.type, data.link,
                          lower_limits,upper_limits,slice_widths,nMC,nBI){
   dCurrent <- FALSE
@@ -105,7 +101,7 @@ calc_a0_func <- function(a0, historical, data.type, data.link,
 #' \enumerate{
 #'
 #' \item	Suppose there are K historical datasets. The user inputs a grid of M rows and K columns of potential values for \eqn{a_0}. For example, one can choose the vector \code{v = c(0.1, 0.25, 0.5, 0.75, 1)}
-#' and use \code{expand.grid(a0_1=v, a0_2=v, a0_3=v)} when K=3 to get a grid with \eqn{M=5^3=125} rows and 3 columns. If there are more than three historical datasets, the dimension of \code{v} can be reduced
+#' and use \code{expand.grid(a0_1=v, a0_2=v, a0_3=v)} when \eqn{K=3} to get a grid with \eqn{M=5^3=125} rows and 3 columns. If there are more than three historical datasets, the dimension of \code{v} can be reduced
 #' to limit the size of the grid. A large grid will increase runtime.
 #' \item	For each row of \eqn{a_0} values in the grid, obtain \eqn{M} samples for \eqn{\beta} from the power prior associated with the current values of \eqn{a_0} using the slice sampler.
 #' \item	For each of the M sets of posterior samples, execute the PWK algorithm (Wang et al., 2018) to estimate the log of normalizing constant \eqn{d_1,...,d_M} for the normalized power prior.
@@ -119,8 +115,8 @@ calc_a0_func <- function(a0, historical, data.type, data.link,
 #' Therefore, it is recommended that \code{grid} values should be at least 0.05.
 #'
 #' If one encounters the error message "some coefficients not defined because of singularities",
-#' it could be due to the following factors: insufficient sample size of the historical data, insufficient number of iterations for the slice sampler,
-#' insufficient number of \code{grid} rows, or near-zero \code{grid} values.
+#' it could be due to the following factors: number of \code{grid} rows too large or too small, insufficient sample size of the historical data, insufficient number of iterations for the slice sampler,
+#' or near-zero \code{grid} values.
 #'
 #'
 #' @return Vector of coefficients for \eqn{a_0} that defines a function \eqn{f(a_0)} that approximates the normalizing constant, necessary for functions \code{\link{glm.random.a0}} and \code{\link{power.glm.random.a0}}.
@@ -163,7 +159,7 @@ normalizing.constant <- function(grid, historical, data.type, data.link,
 
   d <- apply(grid, 1, calc_a0_func, historical, data.type, data.link,
              lower.limits,upper.limits,slice.widths,nMC,nBI) # not a number when a0 too low
-  #print(degree)
+
   m <- as.matrix(grid)
   r2 <- 0
   degree <- 0
@@ -176,7 +172,6 @@ normalizing.constant <- function(grid, historical, data.type, data.link,
       mat[,((i-1)*ncol(m)+1):(ncol(m)*i)] <- m^i
     }
     fit <- lm(d ~ mat)
-    #fit1 <- lm(d ~ a0_1 + a0_2 + I(a0_1^2) + I(a0_2^2) + I(a0_1^3) + I(a0_2^3), data=grid)
     r2 <- summary(fit)$r.squared
 
     if(NA %in% fit$coefficients){
